package uoc.edu.server.dao;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import javax.sql.DataSource;

import org.postgis.PGgeometry;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataAccessException;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Repository;

import uoc.edu.shared.model.Comarca;
import uoc.edu.shared.model.Esdeveniment;
import uoc.edu.shared.model.Poblacio;
import uoc.edu.shared.model.SearchConfig;
import uoc.edu.shared.model.SearchResult;
import uoc.edu.shared.model.TipusEsdeveniment;

import com.extjs.gxt.ui.client.data.BasePagingLoadResult;
import com.extjs.gxt.ui.client.data.PagingLoadConfig;
import com.extjs.gxt.ui.client.data.PagingLoadResult;

/**
 * DAO per el accés a la base de dades per la taula Esdeveniment.
 * 
 * @author Jordi Farré
 * 
 */
@Repository
public class EsdevenimentDAO {

	/** Instancia de JDBC template */
	private JdbcTemplate template;

	/**
	 * Assigna el datasource i crea el jdbd template.
	 * 
	 * @param dataSource
	 *            Instancia del datasource.
	 */
	@Autowired
	public void setDataSource(DataSource dataSource) {
		template = new JdbcTemplate(dataSource);
	}
	
	/**
	 * Retorna tots els esdeveniments de la base de dades.
	 * 
	 * @return Tots els esdeveniments de la base de dades.
	 * @throws Error
	 *             al consultar els esdeveniments.
	 */
	public List<Esdeveniment> getAll() throws DataAccessException {
		String sql = "select idesdeveniment, descripcio esdevenimentdesc from esdeveniment order by esdevenimentdesc asc";
		List<Esdeveniment> result = this.template.query(sql, new RowMapper<Esdeveniment>() {
			@Override
			public Esdeveniment mapRow(ResultSet rs, int rowNum) throws SQLException {
				Esdeveniment esdeveniment = new Esdeveniment();
				esdeveniment.setId(rs.getInt("idesdeveniment"));
				esdeveniment.setDescripcio(rs.getString("esdevenimentdesc"));
				return esdeveniment;
			}
		});
		return result;
	}

	/**
	 * Retorna tots els esdeveniments de la base de dades.
	 * 
	 * @param config
	 *            Configuració de paginació.
	 * @return Tots els esdeveniments de la base de dades.
	 * @throws Error
	 *             al consultar els esdeveniments.
	 */
	public PagingLoadResult<Esdeveniment> getAll(PagingLoadConfig config) throws DataAccessException {
		String countSql = "select count(*) from esdeveniment";
		int count = this.template.queryForInt(countSql);
		String sortField = config.getSortInfo().getSortField();
		String sortDirection = config.getSortInfo().getSortDir().name();
		int start = config.getOffset();
		int limit = count;
		if (config.getLimit() > 0) {
			limit = Math.min(start + config.getLimit(), limit);
		}
		String sql = "select e.idesdeveniment idesdeveniment, t.idtipusesdeveniment idtipusesdeveniment, t.descripcio tipusesdevenimentdesc, e.datainici datainici, e.datafi datafi, e.adreca adreca, e.descripcio descripcio, c.idcomarca idcomarca, c.descripcio comarcadesc, p.idpoblacio idpoblacio, p.descripcio poblaciodesc from esdeveniment e inner join tipusesdeveniment t on e.idtipusesdeveniment = t.idtipusesdeveniment inner join comarca c on e.idcomarca = c.idcomarca inner join poblacio p on e.idpoblacio = p.idpoblacio";
		if (sortField != null) {
			sql += " order by " + sortField + " " + sortDirection;
		}
		sql += " limit " + limit + " offset " + start;
		List<Esdeveniment> result = this.template.query(sql, new RowMapper<Esdeveniment>() {
			@Override
			public Esdeveniment mapRow(ResultSet rs, int rowNum) throws SQLException {
				Esdeveniment esdeveniment = new Esdeveniment();
				esdeveniment.setId(rs.getInt("idesdeveniment"));
				TipusEsdeveniment te = new TipusEsdeveniment();
				te.setId(rs.getInt("idtipusesdeveniment"));
				te.setDescripcio(rs.getString("tipusesdevenimentdesc"));
				esdeveniment.setTipusEsdeveniment(te);
				esdeveniment.setDataInici(rs.getDate("datainici"));
				esdeveniment.setDataFi(rs.getDate("datafi"));
				esdeveniment.setAdreca(rs.getString("adreca"));
				esdeveniment.setDescripcio(rs.getString("descripcio"));
				Comarca com = new Comarca();
				com.setId(rs.getInt("idcomarca"));
				com.setDescripcio(rs.getString("comarcadesc"));
				esdeveniment.setComarca(com);
				Poblacio pob = new Poblacio();
				pob.setId(rs.getInt("idpoblacio"));
				pob.setDescripcio(rs.getString("poblaciodesc"));
				esdeveniment.setPoblacio(pob);
				return esdeveniment;
			}
		});
		return new BasePagingLoadResult<Esdeveniment>(result, start, count);
	}

	/**
	 * Esborra un esdeveniment de la base de dades.
	 * 
	 * @param esdeveniment
	 *            Esdeveniment a esborrar.
	 * @throws DataAccessException
	 *             Error al esborrar el esdeveniment.
	 */
	public void delete(Esdeveniment esdeveniment) throws DataAccessException {
		if (esdeveniment != null) {
			String sql = "delete from esdeveniment where idesdeveniment = ?";
			this.template.update(sql, esdeveniment.getId());
		}
	}
	
	/**
	 * Retorna una llista de marcadors per la cerca.
	 * 
	 * @param searchConfig
	 *            Criteris de cerca.
	 * @return Llista de marcadors.
	 * @throws DataAccessException
	 *             Error de bbdd.
	 */
	public List<SearchResult> getAllMarkers(SearchConfig searchConfig) throws DataAccessException {
		List<SearchResult> result = null;		
		if (searchConfig.getTipusEsdeveniment().getId() != TipusEsdeveniment.CAP) {
			String sql = "select localitzacio, descripcio from esdeveniment";
			if (searchConfig.getComarca().getId() != Comarca.TOTS) {
				if (sql.contains("where")) {
					sql += " and idcomarca = " + searchConfig.getComarca().getId();
				} else {
					sql += " where idcomarca = " + searchConfig.getComarca().getId();
				}
			}
			if (searchConfig.getPoblacio().getId() != Poblacio.TOTS) {
				if (sql.contains("where")) {
					sql += " and idpoblacio = " + searchConfig.getPoblacio().getId();
				} else {
					sql += " where idpoblacio = " + searchConfig.getPoblacio().getId();
				}
			}
			if (searchConfig.getTipusEsdeveniment().getId() != TipusEsdeveniment.TOTS) {
				if (sql.contains("where")) {
					sql += " and idtipusesdeveniment = " + searchConfig.getTipusEsdeveniment().getId();
				} else {
					sql += " where idtipusesdeveniment = " + searchConfig.getTipusEsdeveniment().getId();
				}
			}
			if (searchConfig.getEsdeveniment().getId() != Esdeveniment.TOTS) {
				if (sql.contains("where")) {
					sql += " and idesdeveniment = " + searchConfig.getEsdeveniment().getId();
				} else {
					sql += " where idesdeveniment = " + searchConfig.getEsdeveniment().getId();
				}
			}
			result = this.template.query(sql, new RowMapper<SearchResult>() {
				@Override
				public SearchResult mapRow(ResultSet rs, int arg1) throws SQLException {
					PGgeometry geometry = (PGgeometry) rs.getObject("localitzacio");
					String descripcio = rs.getString("descripcio");
					SearchResult result = new SearchResult();
					result.setTipus(SearchResult.ESDEVENIMENT);
					result.setDescripcio(descripcio);
					result.setLon(geometry.getGeometry().getFirstPoint().getX());
					result.setLat(geometry.getGeometry().getFirstPoint().getY());
					return result;
				}
			});
		} else {
			result = new ArrayList<SearchResult>();
		}
		return result;
	}

}
