package uoc.edu.server.dao;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;

import javax.sql.DataSource;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataAccessException;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Repository;

import uoc.edu.shared.model.Producte;

import com.extjs.gxt.ui.client.data.BasePagingLoadResult;
import com.extjs.gxt.ui.client.data.PagingLoadConfig;
import com.extjs.gxt.ui.client.data.PagingLoadResult;

/**
 * DAO per l'acces a la taula PRODUCTE.
 * 
 * @author Jordi Farré.
 * 
 */
@Repository
public class ProducteDAO {

	/** Instancia de JDBC template */
	private JdbcTemplate template;

	/**
	 * Assigna el datasource i crea el jdbd template.
	 * 
	 * @param dataSource
	 *            Instancia del datasource.
	 */
	@Autowired
	public void setDataSource(DataSource dataSource) {
		template = new JdbcTemplate(dataSource);
	}

	/**
	 * Retorna tots els productes de la base de dades.
	 * 
	 * @return Tots els productes de la base de dades.
	 * @throws Error
	 *             al consultar els productes.
	 */
	public List<Producte> getAll() throws DataAccessException {
		String sql = "select idproducte, descripcio productedesc from producte order by productedesc asc";
		List<Producte> result = this.template.query(sql, new RowMapper<Producte>() {
			@Override
			public Producte mapRow(ResultSet rs, int rowNum) throws SQLException {
				Producte producte = new Producte();
				producte.setId(rs.getInt("idproducte"));
				producte.setDescripcio(rs.getString("productedesc"));
				return producte;
			}
		});
		return result;
	}

	/**
	 * Retorna tots els productes de la base de dades.
	 * 
	 * @param config
	 *            Configuració de paginació.
	 * @return Tots els productes de la base de dades.
	 * @throws Error
	 *             al consultar els productes.
	 */
	public PagingLoadResult<Producte> getAll(PagingLoadConfig config) throws DataAccessException {
		String countSql = "select count(*) from producte";
		int count = this.template.queryForInt(countSql);
		String sortField = config.getSortInfo().getSortField();
		String sortDirection = config.getSortInfo().getSortDir().name();
		int start = config.getOffset();
		int limit = count;
		if (config.getLimit() > 0) {
			limit = Math.min(start + config.getLimit(), limit);
		}
		String sql = "select idproducte, descripcio productedesc from producte";
		if (sortField != null) {
			sql += " order by " + sortField + " " + sortDirection;
		}
		sql += " limit " + limit + " offset " + start;
		List<Producte> result = this.template.query(sql, new RowMapper<Producte>() {
			@Override
			public Producte mapRow(ResultSet rs, int rowNum) throws SQLException {
				Producte producte = new Producte();
				producte.setId(rs.getInt("idproducte"));
				producte.setDescripcio(rs.getString("productedesc"));
				return producte;
			}
		});
		return new BasePagingLoadResult<Producte>(result, start, count);
	}

	/**
	 * Guarda un producte, insertant o actualitzant segons calgui.
	 * 
	 * @param producte
	 *            Tipus de producte a guardar.
	 * @throws DataAccessException
	 *             Excepció al guardar el producte.
	 */
	public void save(Producte producte) throws DataAccessException {
		if (producte.getId() == null) {
			String sql = "insert into producte(descripcio) values (?)";
			this.template.update(sql, producte.getDescripcio());
		} else {
			String sql = "update producte set descripcio = ? where idproducte = ?";
			this.template.update(sql, producte.getDescripcio(), producte.getId());
		}

	}

	/**
	 * Esborra un producte de la base de dades.
	 * 
	 * @param producte
	 *            Producte a esborrar.
	 * @throws DataAccessException
	 *             Error al esborrar el producte.
	 */
	public void delete(Producte producte) throws DataAccessException {
		if (producte != null) {
			String sql = "delete from producte where idproducte = ?";
			this.template.update(sql, producte.getId());
		}
	}

}
