package uoc.edu.server.dao;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import javax.sql.DataSource;

import org.postgis.PGgeometry;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataAccessException;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Repository;

import uoc.edu.shared.model.Comarca;
import uoc.edu.shared.model.Esdeveniment;
import uoc.edu.shared.model.Poblacio;
import uoc.edu.shared.model.Producte;
import uoc.edu.shared.model.Productor;
import uoc.edu.shared.model.SearchConfig;
import uoc.edu.shared.model.SearchResult;
import uoc.edu.shared.model.TipusProductor;
import uoc.edu.shared.model.XCS;

import com.extjs.gxt.ui.client.data.BasePagingLoadResult;
import com.extjs.gxt.ui.client.data.PagingLoadConfig;
import com.extjs.gxt.ui.client.data.PagingLoadResult;

/**
 * DAO per el accés a la base de dades per la taula Productor.
 * 
 * @author Jordi Farré
 * 
 */
@Repository
public class ProductorDAO {

	/** Instancia de JDBC template */
	private JdbcTemplate template;

	/**
	 * Assigna el datasource i crea el jdbd template.
	 * 
	 * @param dataSource
	 *            Instancia del datasource.
	 */
	@Autowired
	public void setDataSource(DataSource dataSource) {
		template = new JdbcTemplate(dataSource);
	}

	/**
	 * Retorna tots els productor de la base de dades.
	 * 
	 * @param config
	 *            Configuració de paginació.
	 * @return Tots els productor de la base de dades.
	 * @throws Error
	 *             al consultar els productors.
	 */
	public PagingLoadResult<Productor> getAll(PagingLoadConfig config) throws DataAccessException {
		String countSql = "select count(*) from productor";
		int count = this.template.queryForInt(countSql);
		String sortField = config.getSortInfo().getSortField();
		String sortDirection = config.getSortInfo().getSortDir().name();
		int start = config.getOffset();
		int limit = count;
		if (config.getLimit() > 0) {
			limit = Math.min(start + config.getLimit(), limit);
		}
		String sql = "select p.idproductor idproductor, t.idtipusproductor idtipusproductor, t.descripcio tipusproductordesc, p.adreca adreca, p.descripcio descripcio, p.telefon telefon, p.email email, c.idcomarca idcomarca, c.descripcio comarcadesc, pob.idpoblacio idpoblacio, pob.descripcio poblaciodesc from productor p inner join tipusproductor t on p.idtipusproductor = t.idtipusproductor inner join comarca c on p.idcomarca = c.idcomarca inner join poblacio pob on p.idpoblacio = pob.idpoblacio";
		if (sortField != null) {
			sql += " order by " + sortField + " " + sortDirection;
		}
		sql += " limit " + limit + " offset " + start;
		List<Productor> result = this.template.query(sql, new RowMapper<Productor>() {
			@Override
			public Productor mapRow(ResultSet rs, int rowNum) throws SQLException {
				Productor productor = new Productor();
				productor.setId(rs.getInt("idproductor"));
				TipusProductor tp = new TipusProductor();
				tp.setId(rs.getInt("idtipusproductor"));
				tp.setDescripcio(rs.getString("tipusproductordesc"));
				productor.setTipusProductor(tp);
				productor.setAdreca(rs.getString("adreca"));
				productor.setDescripcio(rs.getString("descripcio"));
				productor.setTelefon(rs.getString("telefon"));
				productor.setEmail(rs.getString("email"));
				Comarca com = new Comarca();
				com.setId(rs.getInt("idcomarca"));
				com.setDescripcio(rs.getString("comarcadesc"));
				productor.setComarca(com);
				Poblacio pob = new Poblacio();
				pob.setId(rs.getInt("idpoblacio"));
				pob.setDescripcio(rs.getString("poblaciodesc"));
				productor.setPoblacio(pob);
				return productor;
			}
		});
		return new BasePagingLoadResult<Productor>(result, start, count);
	}

	/**
	 * Esborra un productor de la base de dades.
	 * 
	 * @param productor
	 *            Productor a esborrar.
	 * @throws DataAccessException
	 *             Error al esborrar el productor.
	 */
	public void delete(Productor productor) throws DataAccessException {
		if (productor != null) {
			String sql = "delete from productor where idproductor = ?";
			this.template.update(sql, productor.getId());
		}
	}

	/**
	 * Retorna una llista de marcadors per la cerca.
	 * 
	 * @param searchConfig
	 *            Criteris de cerca.
	 * @return Llista de marcadors.
	 * @throws DataAccessException
	 *             Error de bbdd.
	 */
	public List<SearchResult> getAllMarkers(SearchConfig searchConfig) throws DataAccessException {
		List<SearchResult> result = null;
		if (searchConfig.getTipusProductor().getId() != TipusProductor.CAP) {
			String sql = "select p.localitzacio, p.descripcio from productor p "
					+ "left join productor_esdeveniment pe on p.idproductor = pe.idproductor "
					+ "left join esdeveniment e on pe.idesdeveniment = e.idesdeveniment "
					+ "left join productor_producte pp on p.idproductor = pp.idproductor "
					+ "left join xcs_productor xp on p.idproductor = xp.idproductor";
			if (searchConfig.getComarca().getId() != Comarca.TOTS) {
				if (sql.contains("where")) {
					sql += " and p.idcomarca = " + searchConfig.getComarca().getId();
				} else {
					sql += " where p.idcomarca = " + searchConfig.getComarca().getId();
				}
			}
			if (searchConfig.getPoblacio().getId() != Poblacio.TOTS) {
				if (sql.contains("where")) {
					sql += " and p.idpoblacio = " + searchConfig.getPoblacio().getId();
				} else {
					sql += " where p.idpoblacio = " + searchConfig.getPoblacio().getId();
				}
			}
			if (searchConfig.getTipusProductor().getId() != TipusProductor.TOTS) {
				if (sql.contains("where")) {
					sql += " and p.idtipusproductor = " + searchConfig.getTipusProductor().getId();
				} else {
					sql += " where p.idtipusproductor = " + searchConfig.getTipusProductor().getId();
				}
			}
			if (searchConfig.getEsdeveniment().getId() != Esdeveniment.TOTS) {
				if (sql.contains("where")) {
					sql += " and e.idesdeveniment = " + searchConfig.getEsdeveniment().getId();
				} else {
					sql += " where e.idesdeveniment = " + searchConfig.getEsdeveniment().getId();
				}
			}
			if (searchConfig.getProducte().getId() != Producte.TOTS) {
				if (sql.contains("where")) {
					sql += " and pp.idproducte = " + searchConfig.getProducte().getId();
				} else {
					sql += " where pp.idproducte = " + searchConfig.getProducte().getId();
				}
			}
			if (searchConfig.getXcs().getId() != XCS.TOTS) {
				if (sql.contains("where")) {
					sql += " and xp.idxcs = " + searchConfig.getXcs().getId();
				} else {
					sql += " where xp.idxcs = " + searchConfig.getXcs().getId();
				}
			}
			result = this.template.query(sql, new RowMapper<SearchResult>() {
				@Override
				public SearchResult mapRow(ResultSet rs, int arg1) throws SQLException {
					PGgeometry geometry = (PGgeometry) rs.getObject("localitzacio");
					String descripcio = rs.getString("descripcio");
					SearchResult result = new SearchResult();
					result.setTipus(SearchResult.PRODUCTOR);
					result.setDescripcio(descripcio);
					result.setLon(geometry.getGeometry().getFirstPoint().getX());
					result.setLat(geometry.getGeometry().getFirstPoint().getY());
					return result;
				}
			});
		} else {
			result = new ArrayList<SearchResult>();
		}
		return result;
	}

}
