package uoc.edu.server.dao;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import javax.sql.DataSource;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataAccessException;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Repository;

import uoc.edu.shared.model.Esdeveniment;

/**
 * DAO per el accés a la base de dades per la taula Productor_esdeveniment.
 * 
 * @author Jordi Farré
 * 
 */
@Repository
public class ProductorEsdevenimentDAO {

	/** Instancia de JDBC template */
	private JdbcTemplate template;

	/**
	 * Assigna el datasource i crea el jdbd template.
	 * 
	 * @param dataSource
	 *            Instancia del datasource.
	 */
	@Autowired
	public void setDataSource(DataSource dataSource) {
		template = new JdbcTemplate(dataSource);
	}

	/**
	 * Retorna els esdeveniments disponibles per a un determinat productor.
	 * 
	 * @param idProductor
	 *            Id del productor.
	 * @return Llista de esdeveniments disponibles.
	 */
	public List<Esdeveniment> getAllSource(Integer idProductor) {
		String sql = null;
		List<Esdeveniment> result = null;
		if (idProductor != null) {
			sql = "select e.idesdeveniment idesdeveniment, e.descripcio descripcio from esdeveniment e where e.idesdeveniment not in(select pe.idesdeveniment from productor_esdeveniment pe where pe.idproductor = ?) order by e.descripcio asc";
			result = this.template.query(sql, new EsdevenimentRowMapper(), idProductor);
		} else {
			sql = "select e.idesdeveniment idesdeveniment, e.descripcio descripcio from esdeveniment e order by e.descripcio asc";
			result = this.template.query(sql, new EsdevenimentRowMapper());
		}
		return result;
	}

	/**
	 * Retorna tots els esdeveniments associats a un productor.
	 * 
	 * @param idProductor
	 *            Id del productor.
	 * @return Esdeveniments asociats a un productor.
	 */
	public List<Esdeveniment> getAllTarget(Integer idProductor) throws DataAccessException {
		String sql = null;
		List<Esdeveniment> result = null;
		if (idProductor != null) {
			sql = "select e.idesdeveniment idesdeveniment, e.descripcio descripcio from esdeveniment e where e.idesdeveniment in(select pe.idesdeveniment from productor_esdeveniment pe where pe.idproductor = ?) order by e.descripcio asc";
			result = this.template.query(sql, new EsdevenimentRowMapper(), idProductor);
		} else {
			result = new ArrayList<Esdeveniment>();
		}
		return result;
	}

	/**
	 * Comprova si existeix en la bbdd la combinació de productor i
	 * esdeveniment.
	 * 
	 * @param idProductor
	 *            Identificador del productor.
	 * @param idEsdeveniment
	 *            Identificador del esdeveniment.
	 * @return True/False segons si existeix en bbdd o no.
	 * @throws DataAccessException
	 *             Error al accedir a bbdd.
	 */
	public boolean exists(Integer idProductor, Integer idEsdeveniment) throws DataAccessException {
		boolean result = false;
		String sql = "select count(1) from productor_esdeveniment where idproductor = ? and idesdeveniment = ?";
		int count = this.template.queryForInt(sql, idProductor, idEsdeveniment);
		result = count > 0;
		return result;
	}

	/**
	 * Esborra una combinació de productor i esdeveniment.
	 * 
	 * @param idProductor
	 *            Identificador del productor.
	 * @param idEsdeveniment
	 *            Identificador del esdeveniment.
	 * @throws DataAccessException
	 *             Error al accedir a bbdd.
	 */
	public void delete(Integer idProductor, Integer idEsdeveniment) throws DataAccessException {
		String sql = "delete from productor_esdeveniment where idproductor = ? and idesdeveniment = ?";
		this.template.update(sql, idProductor, idEsdeveniment);
	}

	/**
	 * Crea una nova combinació de productor i esdeveniment.
	 * 
	 * @param idProductor
	 *            Identificador del productor.
	 * @param idEsdeveniment
	 *            Identificador del esdeveniment.
	 * @throws DataAccessException
	 *             Error al accedir a bbdd.
	 */
	public void create(Integer idProductor, Integer idEsdeveniment) throws DataAccessException {
		String sql = "insert into productor_esdeveniment(idproductor, idesdeveniment) values (?, ?)";
		this.template.update(sql, idProductor, idEsdeveniment);
	}

	/**
	 * Mapeja els camps de un esdeveniments a partir de un resultset.
	 * 
	 * @author Jordi Farré
	 * 
	 */
	private class EsdevenimentRowMapper implements RowMapper<Esdeveniment> {

		@Override
		public Esdeveniment mapRow(ResultSet rs, int arg1) throws SQLException {
			Esdeveniment esdeveniment = new Esdeveniment();
			esdeveniment.setId(rs.getInt("idesdeveniment"));
			esdeveniment.setDescripcio(rs.getString("descripcio"));
			return esdeveniment;
		}

	}

}
