package uoc.edu.server.dao;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import javax.sql.DataSource;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataAccessException;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Repository;

import uoc.edu.shared.model.Producte;

/**
 * DAO per el accés a la base de dades per la taula Productor_esdeveniment.
 * 
 * @author Jordi Farré
 * 
 */
@Repository
public class ProductorProducteDAO {

	/** Instancia de JDBC template */
	private JdbcTemplate template;

	/**
	 * Assigna el datasource i crea el jdbd template.
	 * 
	 * @param dataSource
	 *            Instancia del datasource.
	 */
	@Autowired
	public void setDataSource(DataSource dataSource) {
		template = new JdbcTemplate(dataSource);
	}

	/**
	 * Retorna els productes disponibles per a un determinat productor.
	 * 
	 * @param idProductor
	 *            Id del productor.
	 * @return Llista de productes disponibles.
	 */
	public List<Producte> getAllSource(Integer idProductor) {
		String sql = null;
		List<Producte> result = null;
		if (idProductor != null) {
			sql = "select e.idproducte idproducte, e.descripcio descripcio from producte e where e.idproducte not in(select pe.idproducte from productor_producte pe where pe.idproductor = ?) order by e.descripcio asc";
			result = this.template.query(sql, new ProducteRowMapper(), idProductor);
		} else {
			sql = "select e.idproducte idproducte, e.descripcio descripcio from producte e order by e.descripcio asc";
			result = this.template.query(sql, new ProducteRowMapper());
		}
		return result;
	}

	/**
	 * Retorna tots els esdeveniments associats a un productor.
	 * 
	 * @param idProductor
	 *            Id del productor.
	 * @return Esdeveniments asociats a un productor.
	 */
	public List<Producte> getAllTarget(Integer idProductor) throws DataAccessException {
		String sql = null;
		List<Producte> result = null;
		if (idProductor != null) {
			sql = "select e.idproducte idproducte, e.descripcio descripcio from producte e where e.idproducte in(select pe.idproducte from productor_producte pe where pe.idproductor = ?) order by e.descripcio asc";
			result = this.template.query(sql, new ProducteRowMapper(), idProductor);
		} else {
			result = new ArrayList<Producte>();
		}
		return result;
	}

	/**
	 * Comprova si existeix en la bbdd la combinació de productor i producte.
	 * 
	 * @param idProductor
	 *            Identificador del productor.
	 * @param idProducte
	 *            Identificador del producte.
	 * @return True/False segons si existeix en bbdd o no.
	 * @throws DataAccessException
	 *             Error al accedir a bbdd.
	 */
	public boolean exists(Integer idProductor, Integer idProducte) throws DataAccessException {
		boolean result = false;
		String sql = "select count(1) from productor_producte where idproductor = ? and idproducte = ?";
		int count = this.template.queryForInt(sql, idProductor, idProducte);
		result = count > 0;
		return result;
	}

	/**
	 * Esborra una combinació de productor i producte.
	 * 
	 * @param idProductor
	 *            Identificador del productor.
	 * @param idProducte
	 *            Identificador del producte.
	 * @throws DataAccessException
	 *             Error al accedir a bbdd.
	 */
	public void delete(Integer idProductor, Integer idProducte) throws DataAccessException {
		String sql = "delete from productor_producte where idproductor = ? and idproducte = ?";
		this.template.update(sql, idProductor, idProducte);
	}

	/**
	 * Crea una nova combinació de productor i producte.
	 * 
	 * @param idProductor
	 *            Identificador del productor.
	 * @param idProducte
	 *            Identificador del producte.
	 * @throws DataAccessException
	 *             Error al accedir a bbdd.
	 */
	public void create(Integer idProductor, Integer idProducte) throws DataAccessException {
		String sql = "insert into productor_producte(idproductor, idproducte) values (?, ?)";
		this.template.update(sql, idProductor, idProducte);
	}

	/**
	 * Mapeja els camps de un producte a partir de un resultset.
	 * 
	 * @author Jordi Farré
	 * 
	 */
	private class ProducteRowMapper implements RowMapper<Producte> {

		@Override
		public Producte mapRow(ResultSet rs, int arg1) throws SQLException {
			Producte producte = new Producte();
			producte.setId(rs.getInt("idproducte"));
			producte.setDescripcio(rs.getString("descripcio"));
			return producte;
		}

	}

}
