package uoc.edu.server.dao;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;

import javax.sql.DataSource;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataAccessException;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Repository;

import uoc.edu.shared.model.TipusEsdeveniment;

import com.extjs.gxt.ui.client.data.BasePagingLoadResult;
import com.extjs.gxt.ui.client.data.PagingLoadConfig;
import com.extjs.gxt.ui.client.data.PagingLoadResult;

/**
 * DAO per el accés a la base de dades per la taula TipusEsdeveniment.
 * 
 * @author Jordi Farré
 * 
 */
@Repository
public class TipusEsdevenimentDAO {

	/** Instancia de JDBC template */
	private JdbcTemplate template;

	/**
	 * Assigna el datasource i crea el jdbd template.
	 * 
	 * @param dataSource
	 *            Instancia del datasource.
	 */
	@Autowired
	public void setDataSource(DataSource dataSource) {
		template = new JdbcTemplate(dataSource);
	}

	/**
	 * Retorna tots els tipus de esdeveniments de la base de dades.
	 * 
	 * @return Tots els tipus de esdeveniments de la base de dades.
	 * @throws Error
	 *             al consultar els tipus de esdeveniments.
	 */
	public List<TipusEsdeveniment> getAll() throws DataAccessException {
		String sql = "select idtipusesdeveniment, descripcio tipusesdevenimentdesc from tipusesdeveniment order by tipusesdevenimentdesc asc";
		List<TipusEsdeveniment> result = this.template.query(sql, new RowMapper<TipusEsdeveniment>() {
			@Override
			public TipusEsdeveniment mapRow(ResultSet rs, int rowNum) throws SQLException {
				TipusEsdeveniment tipusEsdeveniment = new TipusEsdeveniment();
				tipusEsdeveniment.setId(rs.getInt("idtipusesdeveniment"));
				tipusEsdeveniment.setDescripcio(rs.getString("tipusesdevenimentdesc"));
				return tipusEsdeveniment;
			}
		});
		return result;
	}

	/**
	 * Retorna tots els tipus de esdeveniments de la base de dades.
	 * 
	 * @param config
	 *            Configuració de paginació.
	 * @return Tots els tipus de esdeveniments de la base de dades.
	 * @throws Error
	 *             al consultar els tipus de esdeveniments.
	 */
	public PagingLoadResult<TipusEsdeveniment> getAll(PagingLoadConfig config) throws DataAccessException {
		String countSql = "select count(*) from tipusesdeveniment";
		int count = this.template.queryForInt(countSql);
		String sortField = config.getSortInfo().getSortField();
		String sortDirection = config.getSortInfo().getSortDir().name();
		int start = config.getOffset();
		int limit = count;
		if (config.getLimit() > 0) {
			limit = Math.min(start + config.getLimit(), limit);
		}
		String sql = "select idtipusesdeveniment, descripcio tipusesdevenimentdesc from tipusesdeveniment";
		if (sortField != null) {
			sql += " order by " + sortField + " " + sortDirection;
		}
		sql += " limit " + limit + " offset " + start;
		List<TipusEsdeveniment> result = this.template.query(sql, new RowMapper<TipusEsdeveniment>() {
			@Override
			public TipusEsdeveniment mapRow(ResultSet rs, int rowNum) throws SQLException {
				TipusEsdeveniment tipusEsdeveniment = new TipusEsdeveniment();
				tipusEsdeveniment.setId(rs.getInt("idtipusesdeveniment"));
				tipusEsdeveniment.setDescripcio(rs.getString("tipusesdevenimentdesc"));
				return tipusEsdeveniment;
			}
		});
		return new BasePagingLoadResult<TipusEsdeveniment>(result, start, count);
	}

	/**
	 * Guarda un tipus de esdeveniment, insertant o actualitzant segons calgui.
	 * 
	 * @param tipusEsdeveniment
	 *            Tipus d'esdeveniment a guardar.
	 * @throws DataAccessException
	 *             Excepció al guardar el tipus de esdeveniment.
	 */
	public void save(TipusEsdeveniment tipusEsdeveniment) throws DataAccessException {
		if (tipusEsdeveniment.getId() == null) {
			String sql = "insert into tipusesdeveniment(descripcio) values (?)";
			this.template.update(sql, tipusEsdeveniment.getDescripcio());
		} else {
			String sql = "update tipusesdeveniment set descripcio = ? where idtipusesdeveniment = ?";
			this.template.update(sql, tipusEsdeveniment.getDescripcio(), tipusEsdeveniment.getId());
		}

	}

	/**
	 * Esborra un tipus d'esdeveniment de la base de dades.
	 * 
	 * @param tipusEsdeveniment
	 *            Tipus d'esdeveniment a esborrar.
	 * @throws DataAccessException
	 *             Error al esborrar el esdeveniment.
	 */
	public void delete(TipusEsdeveniment tipusEsdeveniment) throws DataAccessException {
		if (tipusEsdeveniment != null) {
			String sql = "delete from tipusesdeveniment where idtipusesdeveniment = ?";
			this.template.update(sql, tipusEsdeveniment.getId());
		}
	}

}
