package uoc.edu.server.dao;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;

import javax.sql.DataSource;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataAccessException;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Repository;

import uoc.edu.shared.model.TipusProductor;

import com.extjs.gxt.ui.client.data.BasePagingLoadResult;
import com.extjs.gxt.ui.client.data.PagingLoadConfig;
import com.extjs.gxt.ui.client.data.PagingLoadResult;

/**
 * DAO per el accés a la base de dades per la taula TipusProductor.
 * 
 * @author Jordi Farré
 * 
 */
@Repository
public class TipusProductorDAO {

	/** Instancia de JDBC template */
	private JdbcTemplate template;

	/**
	 * Assigna el datasource i crea el jdbd template.
	 * 
	 * @param dataSource
	 *            Instancia del datasource.
	 */
	@Autowired
	public void setDataSource(DataSource dataSource) {
		template = new JdbcTemplate(dataSource);
	}

	/**
	 * Retorna tots els tipus de productors de la base de dades.
	 * 
	 * @return Tots els tipus de productors de la base de dades.
	 * @throws Error
	 *             al consultar els tipus de productors.
	 */
	public List<TipusProductor> getAll() throws DataAccessException {
		String sql = "select idtipusproductor, descripcio tipusproductordesc from tipusproductor order by tipusproductordesc asc";
		List<TipusProductor> result = this.template.query(sql, new RowMapper<TipusProductor>() {
			@Override
			public TipusProductor mapRow(ResultSet rs, int rowNum) throws SQLException {
				TipusProductor tipusProductor = new TipusProductor();
				tipusProductor.setId(rs.getInt("idtipusproductor"));
				tipusProductor.setDescripcio(rs.getString("tipusproductordesc"));
				return tipusProductor;
			}
		});
		return result;
	}

	/**
	 * Retorna tots els tipus de productors de la base de dades.
	 * 
	 * @param config
	 *            Configuració de paginació.
	 * @return Tots els tipus de productors de la base de dades.
	 * @throws Error
	 *             al consultar els tipus de productors.
	 */
	public PagingLoadResult<TipusProductor> getAll(PagingLoadConfig config) throws DataAccessException {
		String countSql = "select count(*) from tipusproductor";
		int count = this.template.queryForInt(countSql);
		String sortField = config.getSortInfo().getSortField();
		String sortDirection = config.getSortInfo().getSortDir().name();
		int start = config.getOffset();
		int limit = count;
		if (config.getLimit() > 0) {
			limit = Math.min(start + config.getLimit(), limit);
		}
		String sql = "select idtipusproductor, descripcio tipusproductordesc from tipusproductor";
		if (sortField != null) {
			sql += " order by " + sortField + " " + sortDirection;
		}
		sql += " limit " + limit + " offset " + start;
		List<TipusProductor> result = this.template.query(sql, new RowMapper<TipusProductor>() {
			@Override
			public TipusProductor mapRow(ResultSet rs, int rowNum) throws SQLException {
				TipusProductor tipusProductor = new TipusProductor();
				tipusProductor.setId(rs.getInt("idtipusproductor"));
				tipusProductor.setDescripcio(rs.getString("tipusproductordesc"));
				return tipusProductor;
			}
		});
		return new BasePagingLoadResult<TipusProductor>(result, start, count);
	}

	/**
	 * Guarda un tipus de productor, insertant o actualitzant segons calgui.
	 * 
	 * @param tipusProductor
	 *            Tipus de productor a guardar.
	 * @throws DataAccessException
	 *             Excepció al guardar el tipus de productor.
	 */
	public void save(TipusProductor tipusProductor) throws DataAccessException {
		if (tipusProductor.getId() == null) {
			String sql = "insert into tipusproductor(descripcio) values (?)";
			this.template.update(sql, tipusProductor.getDescripcio());
		} else {
			String sql = "update tipusproductor set descripcio = ? where idtipusproductor = ?";
			this.template.update(sql, tipusProductor.getDescripcio(), tipusProductor.getId());
		}

	}

	/**
	 * Esborra un tipus de productor de la base de dades.
	 * 
	 * @param tipusProductor
	 *            Tipus de productor a esborrar.
	 * @throws DataAccessException
	 *             Error al esborrar el tipus de productor.
	 */
	public void delete(TipusProductor tipusProductor) throws DataAccessException {
		if (tipusProductor != null) {
			String sql = "delete from tipusproductor where idtipusproductor = ?";
			this.template.update(sql, tipusProductor.getId());
		}
	}

}
