package uoc.edu.server.dao;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import javax.sql.DataSource;

import org.postgis.PGgeometry;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataAccessException;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Repository;

import uoc.edu.shared.model.Comarca;
import uoc.edu.shared.model.Poblacio;
import uoc.edu.shared.model.SearchConfig;
import uoc.edu.shared.model.SearchResult;
import uoc.edu.shared.model.TipusEsdeveniment;
import uoc.edu.shared.model.XCS;

import com.extjs.gxt.ui.client.data.BasePagingLoadResult;
import com.extjs.gxt.ui.client.data.PagingLoadConfig;
import com.extjs.gxt.ui.client.data.PagingLoadResult;

/**
 * DAO per el accés a la base de dades per la taula XCS.
 * 
 * @author Jordi Farré
 * 
 */
@Repository
public class XcsDAO {

	/** Instancia de JDBC template */
	private JdbcTemplate template;

	/**
	 * Assigna el datasource i crea el jdbd template.
	 * 
	 * @param dataSource
	 *            Instancia del datasource.
	 */
	@Autowired
	public void setDataSource(DataSource dataSource) {
		template = new JdbcTemplate(dataSource);
	}

	/**
	 * Retorna totes les xcs de la base de dades.
	 * 
	 * @return Tots les xcs de la base de dades.
	 * @throws Error
	 *             al consultar les xcs.
	 */
	public List<XCS> getAll() throws DataAccessException {
		String sql = "select idxcs, descripcio xcsdesc from xcs order by xcsdesc asc";
		List<XCS> result = this.template.query(sql, new RowMapper<XCS>() {
			@Override
			public XCS mapRow(ResultSet rs, int rowNum) throws SQLException {
				XCS xcs = new XCS();
				xcs.setId(rs.getInt("idxcs"));
				xcs.setDescripcio(rs.getString("xcsdesc"));
				return xcs;
			}
		});
		return result;
	}

	/**
	 * Retorna tots les xcs de la base de dades.
	 * 
	 * @param config
	 *            Configuració de paginació.
	 * @return Tots les xcs de la base de dades.
	 * @throws Error
	 *             al consultar les xcs.
	 */
	public PagingLoadResult<XCS> getAll(PagingLoadConfig config) throws DataAccessException {
		String countSql = "select count(*) from xcs";
		int count = this.template.queryForInt(countSql);
		String sortField = config.getSortInfo().getSortField();
		String sortDirection = config.getSortInfo().getSortDir().name();
		int start = config.getOffset();
		int limit = count;
		if (config.getLimit() > 0) {
			limit = Math.min(start + config.getLimit(), limit);
		}
		String sql = "select x.idxcs idxcs, x.adreca adreca, x.descripcio descripcio, x.telefon telefon, x.email email, c.idcomarca idcomarca, c.descripcio comarcadesc, pob.idpoblacio idpoblacio, pob.descripcio poblaciodesc from xcs x inner join comarca c on x.idcomarca = c.idcomarca inner join poblacio pob on x.idpoblacio = pob.idpoblacio";
		if (sortField != null) {
			sql += " order by " + sortField + " " + sortDirection;
		}
		sql += " limit " + limit + " offset " + start;
		List<XCS> result = this.template.query(sql, new RowMapper<XCS>() {
			@Override
			public XCS mapRow(ResultSet rs, int rowNum) throws SQLException {
				XCS xcs = new XCS();
				xcs.setId(rs.getInt("idxcs"));
				xcs.setAdreca(rs.getString("adreca"));
				xcs.setDescripcio(rs.getString("descripcio"));
				xcs.setTelefon(rs.getString("telefon"));
				xcs.setEmail(rs.getString("email"));
				Comarca com = new Comarca();
				com.setId(rs.getInt("idcomarca"));
				com.setDescripcio(rs.getString("comarcadesc"));
				xcs.setComarca(com);
				Poblacio pob = new Poblacio();
				pob.setId(rs.getInt("idpoblacio"));
				pob.setDescripcio(rs.getString("poblaciodesc"));
				xcs.setPoblacio(pob);
				return xcs;
			}
		});
		return new BasePagingLoadResult<XCS>(result, start, count);
	}

	/**
	 * Esborra una xcs de la base de dades.
	 * 
	 * @param xcs
	 *            XCS a esborrar.
	 * @throws DataAccessException
	 *             Error al esborrar la xcs.
	 */
	public void delete(XCS xcs) throws DataAccessException {
		if (xcs != null) {
			String sql = "delete from xcs where idxcs = ?";
			this.template.update(sql, xcs.getId());
		}
	}

	/**
	 * Retorna una llista de marcadors per la cerca.
	 * 
	 * @param searchConfig
	 *            Criteris de cerca.
	 * @return Llista de marcadors.
	 * @throws DataAccessException
	 *             Error de bbdd.
	 */
	public List<SearchResult> getAllMarkers(SearchConfig searchConfig) throws DataAccessException {
		List<SearchResult> result = null;
		String sql = "select localitzacio, descripcio from xcs";
		if (searchConfig.getXcs().getId() != XCS.TOTS) {
			if (sql.contains("where")) {
				sql += " and idxcs = " + searchConfig.getXcs().getId();
			} else {
				sql += " where idxcs = " + searchConfig.getXcs().getId();
			}
		}
		if (searchConfig.getComarca().getId() != Comarca.TOTS) {
			if (sql.contains("where")) {
				sql += " and idcomarca = " + searchConfig.getComarca().getId();
			} else {
				sql += " where idcomarca = " + searchConfig.getComarca().getId();
			}
		}
		if (searchConfig.getPoblacio().getId() != Poblacio.TOTS) {
			if (sql.contains("where")) {
				sql += " and idpoblacio = " + searchConfig.getPoblacio().getId();
			} else {
				sql += " where idpoblacio = " + searchConfig.getPoblacio().getId();
			}
		}
		result = this.template.query(sql, new RowMapper<SearchResult>() {
			@Override
			public SearchResult mapRow(ResultSet rs, int arg1) throws SQLException {
				PGgeometry geometry = (PGgeometry) rs.getObject("localitzacio");
				String descripcio = rs.getString("descripcio");
				SearchResult result = new SearchResult();
				result.setTipus(SearchResult.XCS);
				result.setDescripcio(descripcio);
				result.setLon(geometry.getGeometry().getFirstPoint().getX());
				result.setLat(geometry.getGeometry().getFirstPoint().getY());
				return result;
			}
		});
		return result;
	}

}
