package uoc.edu.server.dao;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import javax.sql.DataSource;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataAccessException;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Repository;

import uoc.edu.shared.model.Productor;

/**
 * DAO per el accés a la base de dades per la taula xcs_productor.
 * 
 * @author Jordi Farré
 * 
 */
@Repository
public class XcsProductorDAO {

	/** Instancia de JDBC template */
	private JdbcTemplate template;

	/**
	 * Assigna el datasource i crea el jdbc template.
	 * 
	 * @param dataSource
	 *            Instancia del datasource.
	 */
	@Autowired
	public void setDataSource(DataSource dataSource) {
		template = new JdbcTemplate(dataSource);
	}

	/**
	 * Retorna els productors disponibles per a una determinada xcs.
	 * 
	 * @param idXcs
	 *            Id de la xcs.
	 * @return Llista de productors disponibles.
	 */
	public List<Productor> getAllSource(Integer idXcs) {
		String sql = null;
		List<Productor> result = null;
		if (idXcs != null) {
			sql = "select p.idproductor idproductor, p.descripcio descripcio from productor p where p.idproductor not in(select xp.idproductor from xcs_productor xp where xp.idxcs = ?) order by p.descripcio asc";
			result = this.template.query(sql, new ProductorRowMapper(), idXcs);
		} else {
			sql = "select p.idproductor idproductor, p.descripcio descripcio from productor p order by p.descripcio asc";
			result = this.template.query(sql, new ProductorRowMapper());
		}
		return result;
	}

	/**
	 * Retorna tots els productors associats a una xcs.
	 * 
	 * @param idXcs
	 *            Id de la xcs.
	 * @return Productors asociats a una xcs.
	 */
	public List<Productor> getAllTarget(Integer idXcs) throws DataAccessException {
		String sql = null;
		List<Productor> result = null;
		if (idXcs != null) {
			sql = "select p.idproductor idproductor, p.descripcio descripcio from productor p where p.idproductor in(select xp.idproductor from xcs_productor xp where xp.idxcs = ?) order by p.descripcio asc";
			result = this.template.query(sql, new ProductorRowMapper(), idXcs);
		} else {
			result = new ArrayList<Productor>();
		}
		return result;
	}

	/**
	 * Comprova si existeix en la bbdd la combinació de xcs i
	 * productor.
	 * 
	 * @param idXcs
	 *            Identificador de la xcs.
	 * @param idProductor
	 *            Identificador del productor.
	 * @return True/False segons si existeix en bbdd o no.
	 * @throws DataAccessException
	 *             Error al accedir a bbdd.
	 */
	public boolean exists(Integer idXcs, Integer idProductor) throws DataAccessException {
		boolean result = false;
		String sql = "select count(1) from xcs_productor where idxcs = ? and idproductor = ?";
		int count = this.template.queryForInt(sql, idXcs, idProductor);
		result = count > 0;
		return result;
	}

	/**
	 * Esborra una combinació de xcs i productor.
	 * 
	 * @param idXcs
	 *            Identificador de la xcs.
	 * @param idProductor
	 *            Identificador del productor.
	 * @throws DataAccessException
	 *             Error al accedir a bbdd.
	 */
	public void delete(Integer idXcs, Integer idProductor) throws DataAccessException {
		String sql = "delete from xcs_productor where idxcs = ? and idproductor = ?";
		this.template.update(sql, idXcs, idProductor);
	}

	/**
	 * Crea una nova combinació de xcs i productor.
	 * 
	 * @param idXcs
	 *            Identificador del xcs.
	 * @param idProductor
	 *            Identificador del productor.
	 * @throws DataAccessException
	 *             Error al accedir a bbdd.
	 */
	public void create(Integer idXcs, Integer idProductor) throws DataAccessException {
		String sql = "insert into xcs_productor(idxcs, idproductor) values (?, ?)";
		this.template.update(sql, idXcs, idProductor);
	}

	/**
	 * Mapeja els camps de un productor a partir de un resultset.
	 * 
	 * @author Jordi Farré
	 * 
	 */
	private class ProductorRowMapper implements RowMapper<Productor> {

		@Override
		public Productor mapRow(ResultSet rs, int arg1) throws SQLException {
			Productor productor = new Productor();
			productor.setId(rs.getInt("idproductor"));
			productor.setDescripcio(rs.getString("descripcio"));
			return productor;
		}

	}

}
